
import Foundation
import UIKit

enum HipsterElementType: String {
  case Glasses1 = "glasses1"
  case Glasses2 = "glasses2"
  case Fedora1 = "fedora1"
  case Fedora2 = "fedora2"
}

class HipsterElement: NSObject {
  let type: HipsterElementType
  private(set) var relativeCenter: CGPoint
  private(set) var relativeWidth: CGFloat
  var rotation: CGFloat = 0 {
    didSet {
      if let view = self.view {
        view.transform = CGAffineTransformScale(CGAffineTransformMakeRotation(rotation), scale, scale)
      }
    }
  }

  var scale: CGFloat! {
    didSet {
      if let view = self.view {
        view.transform = CGAffineTransformMakeScale(scale, scale)
        relativeWidth = CGRectGetWidth(view.frame) / CGRectGetWidth(imageFrame)
        view.transform = CGAffineTransformScale(CGAffineTransformMakeRotation(rotation), scale, scale)
      }
    }
  }

  var view: UIImageView!

  /// Frame of the actual image being hipsterized
  var imageFrame: CGRect = CGRect.zeroRect

  var active = false

  // passthroughs to view
  var center: CGPoint {
    get { return self.view?.center ?? CGPoint.zeroPoint }
    set {
      view?.center = newValue
      relativeCenter = CGPoint(x: (newValue.x-imageFrame.origin.x) / CGRectGetWidth(imageFrame), y: (newValue.y-imageFrame.origin.y) / CGRectGetHeight(imageFrame))
    }
  }

  init(type: HipsterElementType, relativeCenter: CGPoint, relativeWidth: CGFloat, rotation: CGFloat) {
    self.type = type
    self.relativeCenter = relativeCenter
    self.relativeWidth = relativeWidth
    self.rotation = rotation
  }

  func createViewForImageFrame(imageFrame: CGRect) {
    view = UIImageView(image: UIImage(named: type.rawValue))
    view.contentMode = .ScaleAspectFit

    setViewPositioningForImageFrame(imageFrame)
  }

  func createViewWithFrame(frame: CGRect) {
    view = UIImageView(image: UIImage(named: type.rawValue))
    view.contentMode = .ScaleAspectFit

    scale = CGRectGetWidth(frame) / view.image!.size.width
    view.transform = CGAffineTransformMakeScale(scale, scale)
    view.center = CGPoint(x: frame.origin.x + frame.width/2, y: frame.origin.y + frame.height/2)
  }

  func setViewPositioningForImageFrame(imageFrame: CGRect) {
    self.imageFrame = imageFrame

    if let view = view {
      view.sizeToFit()

      view.center = CGPoint(x: relativeCenter.x * CGRectGetWidth(imageFrame) + imageFrame.origin.x,
        y: relativeCenter.y * CGRectGetHeight(imageFrame) + imageFrame.origin.y)

      let elementWidth = relativeWidth * CGRectGetWidth(imageFrame)
      scale = elementWidth / view.image!.size.width

      view.transform = CGAffineTransformScale(CGAffineTransformMakeRotation(rotation), scale, scale)
    }
  }

  func elementAspectRatio() -> CGFloat {
    return CGRectGetWidth(view.frame) / CGRectGetHeight(view.frame)
  }

  func elementImageAspectRatio() -> CGFloat {
    if let image = view.image {
      return image.size.width / image.size.height
    } else {
      return 1
    }
  }

  func setGlow(glow: Bool) {
    UIView.animateWithDuration(0.2, animations: {
      if glow {
        self.view?.layer.borderColor = UIColor.greenColor().CGColor
        self.view?.layer.borderWidth = 2.0
      } else {
        self.view?.layer.borderWidth = 0
      }
    })
  }
}
